@@ -1,7 +1,7 @@
 package Git::Bunch;
 
-our $DATE = '2015-01-22'; # DATE
-our $VERSION = '0.44'; # VERSION
+our $DATE = '2015-07-25'; # DATE
+our $VERSION = '0.45'; # VERSION
 
 use 5.010001;
 use strict;
@@ -37,17 +37,42 @@ our %common_args_spec = (
         summary      => 'Order entries in bunch',
         description  => <<'_',
 
-`commit-timestamp` (and `-commit-timestamp`) compares the timestamp of
+If sorting is enabled, the repos will be sorted first by some ordering. One
+use-case for this is to allow more recently committed-to repos to be processed
+first (using `-commit-timestamp` or `-db-commit-timestamp`).
+
+`db-commit-timestamp` (or `-db-commit-timestamp`) reads SQLite database file
+`repos.db` in the source directory and to get last commit timestamp information
+(in the `repos` table, the `commit_timestamp` column). You will need to create
+and maintain this database, e.g. in your post-commit script. Repos or dirs not
+having the last commit information in the database will be processed later. This
+method is faster than `commit-timestamp` (or `-commit-timestamp`, see next
+paragraph) if your source directory contains lots (e.g. hundreds or thousands)
+of repos because you avoid having to stat() each `.git/commit-timestamp` file in
+each repo.
+
+`commit-timestamp` (or `-commit-timestamp`) compares the timestamp of
 `.git/commit-timestamp` file in each repo. Repos or dirs not having this file
 will be processed later. You can touch these `.git/commit-timestamp` files in
 your post-commit script, for example. This allows sorting recently committed
 repos more cheaply (compared to doing `git log -1`).
 
+`mtime` (or `-mtime`) compares the timestamp or the repo dirs. This might not
+give the result you want if you expect to process more recently updated repos
+first, because files in a repo might be updated in the subdirectories of the
+repo instead of in the top-level dir.
+
+`name` (or `-name`) simply compares the repos' names. This is one of the fastest
+methods.
+
+`rand` randomizes the order of repos, so you get different ordering in each run.
+
 _
         schema       => ['str' => {
             default => '-commit-timestamp',
             in      => [qw/name -name mtime -mtime rand
                            commit-timestamp -commit-timestamp
+                           db-commit-timestamp -db-commit-timestamp
                           /],
         }],
     },
@@ -161,7 +186,6 @@ sub _check_common_args {
     my $sort = $args->{sort} // "-mtime";
     my $sortsub;
     if (!$sort) {
-        $sortsub = sub { 1 };
     } elsif ($sort eq '-mtime') {
         $sortsub = sub {((-M $a)//0) <=> ((-M $b)//0)};
     } elsif ($sort eq 'mtime') {
@@ -181,6 +205,25 @@ sub _check_common_args {
             return -1 if !$ts_b;
             return $ts_a <=> $ts_b;
         };
+    } elsif ($sort =~ /^(-)db-commit-timestamp$/) {
+        my $rev = $1;
+        my $db_path = "$args->{source}/repos.db";
+        (-f $db_path) or die "Can't find '$db_path'";
+        require DBI;
+        my $dbh = DBI->connect("dbi:SQLite:dbname=$db_path", "", "",
+                               {RaiseError=>1});
+        my %pos; # key=repo name
+        my $sth = $dbh->prepare(
+            "SELECT name FROM repos ORDER BY commit_timestamp".
+                ($rev ? " DESC" : ""));
+        $sth->execute;
+        my $i = 0;
+        while (my ($name) = $sth->fetchrow_array) {
+            $pos{$name} = $i++;
+        }
+        $sortsub = sub {
+            ($pos{$a} // 999_999) <=> ($pos{$b} // 999_999)
+        };
     } else { # rand
         $sortsub = sub {int(3*rand())-1};
     }
@@ -309,7 +352,9 @@ sub check_bunch {
     my %res;
     local $CWD = $source;
 
-    my @entries = sort $sortsub grep {-d} <*>;
+    my @entries;
+    @entries = grep {-d} <*>;
+    @entries = sort $sortsub @entries if $sortsub;
     #$log->tracef("entries: %s", \@entries);
 
     my $i = 0;
@@ -364,7 +409,7 @@ sub check_bunch {
     [200,
      $has_unclean ? "Some repos unclean" : "All repos clean",
      \%res,
-     {"cmdline.result" => ''}];
+     {'func.res'=>\%res}];
 }
 
 sub _sync_repo {
@@ -645,7 +690,8 @@ sub sync_bunch {
     {
         local $CWD = $source;
         opendir my($d), ".";
-        @entries = sort $sortsub readdir($d);
+        @entries = readdir($d);
+        @entries = sort $sortsub @entries if $sortsub;
     }
     #$log->tracef("entries: %s", \@entries);
 
@@ -794,7 +840,8 @@ sub exec_bunch {
     local $CWD = $source;
     my %res;
     my $i = 0;
-    my @entries = sort $sortsub grep {-d} <*>;
+    my @entries = grep {-d} <*>;
+    @entries = sort $sortsub @entries if $sortsub;
     #$log->tracef("entries: %s", \@entries);
   REPO:
     for my $repo (@entries) {
@@ -833,7 +880,7 @@ Git::Bunch - Manage gitbunch directory (directory which contain git repos)
 
 =head1 VERSION
 
-This document describes version 0.44 of Git::Bunch (from Perl distribution Git-Bunch), released on 2015-01-22.
+This document describes version 0.45 of Git::Bunch (from Perl distribution Git-Bunch), released on 2015-07-25.
 
 =head1 SYNOPSIS
 
@@ -879,14 +926,12 @@ See also L<File::RsyBak>, which I wrote to backup everything else.
 
 =head2 check_bunch(%args) -> [status, msg, result, meta]
 
-{en_US Check status of git repositories inside gitbunch directory}.
+Check status of git repositories inside gitbunch directory.
 
-{en_US 
 Will perform a 'git status' for each git repositories inside the bunch and
 report which repositories are clean/unclean.
 
 Will die if can't chdir into bunch or git repository.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -894,57 +939,75 @@ Arguments ('*' denotes required arguments):
 
 =item * B<exclude_files> => I<bool>
 
-{en_US Exclude files from processing}.
+Exclude files from processing.
 
-{en_US 
 This only applies to C<sync_bunch> operations. Operations like C<check_bunch> and
 C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_non_git_dirs> => I<bool>
 
-{en_US Exclude non-git dirs from processing}.
+Exclude non-git dirs from processing.
 
-{en_US 
 This only applies to and C<sync_bunch> operations. Operations like C<check_bunch>
 and C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_repos> => I<array[str]>
 
-{en_US Exclude some repos from processing}.
+Exclude some repos from processing.
 
 =item * B<exclude_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to exclude}.
+Specify regex pattern of repos to exclude.
 
 =item * B<include_repos> => I<array[str]>
 
-{en_US Specific git repos to sync, if not specified all repos in the bunch will be processed}.
+Specific git repos to sync, if not specified all repos in the bunch will be processed.
 
 =item * B<include_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to include}.
+Specify regex pattern of repos to include.
 
 =item * B<repo> => I<str>
 
-{en_US Only process a single repo}.
+Only process a single repo.
 
 =item * B<sort> => I<str> (default: "-commit-timestamp")
 
-{en_US Order entries in bunch}.
+Order entries in bunch.
 
-{en_US 
-C<commit-timestamp> (and C<-commit-timestamp>) compares the timestamp of
+If sorting is enabled, the repos will be sorted first by some ordering. One
+use-case for this is to allow more recently committed-to repos to be processed
+first (using C<-commit-timestamp> or C<-db-commit-timestamp>).
+
+C<db-commit-timestamp> (or C<-db-commit-timestamp>) reads SQLite database file
+C<repos.db> in the source directory and to get last commit timestamp information
+(in the C<repos> table, the C<commit_timestamp> column). You will need to create
+and maintain this database, e.g. in your post-commit script. Repos or dirs not
+having the last commit information in the database will be processed later. This
+method is faster than C<commit-timestamp> (or C<-commit-timestamp>, see next
+paragraph) if your source directory contains lots (e.g. hundreds or thousands)
+of repos because you avoid having to stat() each C<.git/commit-timestamp> file in
+each repo.
+
+C<commit-timestamp> (or C<-commit-timestamp>) compares the timestamp of
 C<.git/commit-timestamp> file in each repo. Repos or dirs not having this file
 will be processed later. You can touch these C<.git/commit-timestamp> files in
 your post-commit script, for example. This allows sorting recently committed
 repos more cheaply (compared to doing C<git log -1>).
-}
+
+C<mtime> (or C<-mtime>) compares the timestamp or the repo dirs. This might not
+give the result you want if you expect to process more recently updated repos
+first, because files in a repo might be updated in the subdirectories of the
+repo instead of in the top-level dir.
+
+C<name> (or C<-name>) simply compares the repos' names. This is one of the fastest
+methods.
+
+C<rand> randomizes the order of repos, so you get different ordering in each run.
 
 =item * B<source>* => I<str>
 
-{en_US Directory to check}.
+Directory to check.
 
 =back
 
@@ -962,12 +1025,10 @@ Return value:  (any)
 
 =head2 exec_bunch(%args) -> [status, msg, result, meta]
 
-{en_US Execute a command for each repo in the bunch}.
+Execute a command for each repo in the bunch.
 
-{en_US 
 For each git repository in the bunch, will chdir to it and execute specified
 command.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -975,61 +1036,79 @@ Arguments ('*' denotes required arguments):
 
 =item * B<command>* => I<str>
 
-{en_US Command to execute}.
+Command to execute.
 
 =item * B<exclude_files> => I<bool>
 
-{en_US Exclude files from processing}.
+Exclude files from processing.
 
-{en_US 
 This only applies to C<sync_bunch> operations. Operations like C<check_bunch> and
 C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_non_git_dirs> => I<bool>
 
-{en_US Exclude non-git dirs from processing}.
+Exclude non-git dirs from processing.
 
-{en_US 
 This only applies to and C<sync_bunch> operations. Operations like C<check_bunch>
 and C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_repos> => I<array[str]>
 
-{en_US Exclude some repos from processing}.
+Exclude some repos from processing.
 
 =item * B<exclude_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to exclude}.
+Specify regex pattern of repos to exclude.
 
 =item * B<include_repos> => I<array[str]>
 
-{en_US Specific git repos to sync, if not specified all repos in the bunch will be processed}.
+Specific git repos to sync, if not specified all repos in the bunch will be processed.
 
 =item * B<include_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to include}.
+Specify regex pattern of repos to include.
 
 =item * B<repo> => I<str>
 
-{en_US Only process a single repo}.
+Only process a single repo.
 
 =item * B<sort> => I<str> (default: "-commit-timestamp")
 
-{en_US Order entries in bunch}.
+Order entries in bunch.
 
-{en_US 
-C<commit-timestamp> (and C<-commit-timestamp>) compares the timestamp of
+If sorting is enabled, the repos will be sorted first by some ordering. One
+use-case for this is to allow more recently committed-to repos to be processed
+first (using C<-commit-timestamp> or C<-db-commit-timestamp>).
+
+C<db-commit-timestamp> (or C<-db-commit-timestamp>) reads SQLite database file
+C<repos.db> in the source directory and to get last commit timestamp information
+(in the C<repos> table, the C<commit_timestamp> column). You will need to create
+and maintain this database, e.g. in your post-commit script. Repos or dirs not
+having the last commit information in the database will be processed later. This
+method is faster than C<commit-timestamp> (or C<-commit-timestamp>, see next
+paragraph) if your source directory contains lots (e.g. hundreds or thousands)
+of repos because you avoid having to stat() each C<.git/commit-timestamp> file in
+each repo.
+
+C<commit-timestamp> (or C<-commit-timestamp>) compares the timestamp of
 C<.git/commit-timestamp> file in each repo. Repos or dirs not having this file
 will be processed later. You can touch these C<.git/commit-timestamp> files in
 your post-commit script, for example. This allows sorting recently committed
 repos more cheaply (compared to doing C<git log -1>).
-}
+
+C<mtime> (or C<-mtime>) compares the timestamp or the repo dirs. This might not
+give the result you want if you expect to process more recently updated repos
+first, because files in a repo might be updated in the subdirectories of the
+repo instead of in the top-level dir.
+
+C<name> (or C<-name>) simply compares the repos' names. This is one of the fastest
+methods.
+
+C<rand> randomizes the order of repos, so you get different ordering in each run.
 
 =item * B<source>* => I<str>
 
-{en_US Directory to check}.
+Directory to check.
 
 =back
 
@@ -1047,16 +1126,14 @@ Return value:  (any)
 
 =head2 sync_bunch(%args) -> [status, msg, result, meta]
 
-{en_US Synchronize bunch to another bunch}.
+Synchronize bunch to another bunch.
 
-{en_US 
 For each git repository in the bunch, will perform a 'git pull/push' for each
 branch. If repository in destination doesn't exist, it will be rsync-ed first
 from source. When 'git pull' fails, will exit to let you fix the problem
 manually.
 
 For all other non-git repos, will simply synchronize by one-way rsync.
-}
 
 Arguments ('*' denotes required arguments):
 
@@ -1064,9 +1141,8 @@ Arguments ('*' denotes required arguments):
 
 =item * B<backup> => I<bool>
 
-{en_US Whether doing backup to target}.
+Whether doing backup to target.
 
-{en_US 
 This setting lets you express that you want to perform synchronizing to a backup
 target, and that you do not do work on the target. Thus, you do not care about
 uncommitted or untracked files/dirs in the target repos (might happen if you
@@ -1075,13 +1151,11 @@ is turned on, the function will first do a C<git clean -f -d> (to delete
 untracked files/dirs) and then C<git checkout .> (to discard all uncommitted
 changes). This setting will also implicitly turn on C<create_bare> setting
 (unless that setting has been explicitly enabled/disabled).
-}
 
 =item * B<create_bare_target> => I<bool>
 
-{en_US Whether to create bare git repo when target does not exist}.
+Whether to create bare git repo when target does not exist.
 
-{en_US 
 When target repo does not exist, gitbunch can either copy the source repo using
 C<rsync> (the default, if this setting is undefined), or it can create target
 repo with C<git init --bare> (if this setting is set to 1), or it can create
@@ -1094,80 +1168,95 @@ Creating bare repos are apt for backup purposes since they are more
 space-efficient.
 
 Non-repos will still be copied/rsync-ed.
-}
 
 =item * B<delete_branch> => I<bool> (default: 0)
 
-{en_US Whether to delete branches in dest repos not existing in source repos}.
+Whether to delete branches in dest repos not existing in source repos.
 
 =item * B<exclude_files> => I<bool>
 
-{en_US Exclude files from processing}.
+Exclude files from processing.
 
-{en_US 
 This only applies to C<sync_bunch> operations. Operations like C<check_bunch> and
 C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_non_git_dirs> => I<bool>
 
-{en_US Exclude non-git dirs from processing}.
+Exclude non-git dirs from processing.
 
-{en_US 
 This only applies to and C<sync_bunch> operations. Operations like C<check_bunch>
 and C<exec_bunch> already ignore these and only operate on git repos.
-}
 
 =item * B<exclude_repos> => I<array[str]>
 
-{en_US Exclude some repos from processing}.
+Exclude some repos from processing.
 
 =item * B<exclude_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to exclude}.
+Specify regex pattern of repos to exclude.
 
 =item * B<include_repos> => I<array[str]>
 
-{en_US Specific git repos to sync, if not specified all repos in the bunch will be processed}.
+Specific git repos to sync, if not specified all repos in the bunch will be processed.
 
 =item * B<include_repos_pat> => I<str>
 
-{en_US Specify regex pattern of repos to include}.
+Specify regex pattern of repos to include.
 
 =item * B<repo> => I<str>
 
-{en_US Only process a single repo}.
+Only process a single repo.
 
 =item * B<rsync_opt_maintain_ownership> => I<bool> (default: 0)
 
-{en_US Whether or not, when rsync-ing from source, we use -a (= -rlptgoD) or -rlptD (-a minus -go)}.
+Whether or not, when rsync-ing from source, we use -a (= -rlptgoD) or -rlptD (-a minus -go).
 
-{en_US 
 Sometimes using -a results in failure to preserve permission modes on
 sshfs-mounted filesystem, while -rlptD succeeds, so by default we don't maintain
 ownership. If you need to maintain ownership (e.g. you run as root and the repos
 are not owned by root), turn this option on.
-}
 
 =item * B<sort> => I<str> (default: "-commit-timestamp")
 
-{en_US Order entries in bunch}.
+Order entries in bunch.
 
-{en_US 
-C<commit-timestamp> (and C<-commit-timestamp>) compares the timestamp of
+If sorting is enabled, the repos will be sorted first by some ordering. One
+use-case for this is to allow more recently committed-to repos to be processed
+first (using C<-commit-timestamp> or C<-db-commit-timestamp>).
+
+C<db-commit-timestamp> (or C<-db-commit-timestamp>) reads SQLite database file
+C<repos.db> in the source directory and to get last commit timestamp information
+(in the C<repos> table, the C<commit_timestamp> column). You will need to create
+and maintain this database, e.g. in your post-commit script. Repos or dirs not
+having the last commit information in the database will be processed later. This
+method is faster than C<commit-timestamp> (or C<-commit-timestamp>, see next
+paragraph) if your source directory contains lots (e.g. hundreds or thousands)
+of repos because you avoid having to stat() each C<.git/commit-timestamp> file in
+each repo.
+
+C<commit-timestamp> (or C<-commit-timestamp>) compares the timestamp of
 C<.git/commit-timestamp> file in each repo. Repos or dirs not having this file
 will be processed later. You can touch these C<.git/commit-timestamp> files in
 your post-commit script, for example. This allows sorting recently committed
 repos more cheaply (compared to doing C<git log -1>).
-}
+
+C<mtime> (or C<-mtime>) compares the timestamp or the repo dirs. This might not
+give the result you want if you expect to process more recently updated repos
+first, because files in a repo might be updated in the subdirectories of the
+repo instead of in the top-level dir.
+
+C<name> (or C<-name>) simply compares the repos' names. This is one of the fastest
+methods.
+
+C<rand> randomizes the order of repos, so you get different ordering in each run.
 
 =item * B<source>* => I<str>
 
-{en_US Directory to check}.
+Directory to check.
 
 =item * B<target>* => I<str>
 
-{en_US Destination bunch}.
+Destination bunch.
 
 =back
 
